/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;

/**
 * Representation of the beginning transaction output builder.
 */
public class BeginTransactionOutputBuilder extends DefaultOutputBuilderBase {

    private Map<Class<? extends BeginTransactionOutput>, BeginTransactionOutput>
            augmentation = new HashMap<>();

    /**
     * Creates a beginning transaction output builder.
     */
    public BeginTransactionOutputBuilder() {
    }

    /**
     * Creates the beginning transaction output builder from a RPC result.
     *
     * @param rpc a common RPC result
     */
    public BeginTransactionOutputBuilder(CommonRpcResult rpc) {
        resultCode = rpc.getResultCode();
        message = rpc.getMessage();
    }

    /**
     * Creates a beginning transaction output builder from
     * a beginning transaction output.
     *
     * @param base beginning transaction output object
     */
    public BeginTransactionOutputBuilder(BeginTransactionOutput base) {
        message = base.getMessage();
        resultCode = base.getResultCode();
        if (base instanceof BeginTransactionOutputImpl) {
            BeginTransactionOutputImpl impl = (BeginTransactionOutputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Adds augmentation by a class type and a beginning transaction output.
     * If the augmentation is null, remove the augmentation type key
     * from the hash map.
     *
     * @param augmentationType class extends BeginTransactionOutput interface
     * @param augmentation a beginning transaction output
     * @return self
     */
    public BeginTransactionOutputBuilder addAugmentation(
            Class<? extends BeginTransactionOutput> augmentationType,
            BeginTransactionOutput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType class extends BeginTransactionOutput interface
     * @return self
     */
    public BeginTransactionOutputBuilder removeAugmentation(
            Class<? extends BeginTransactionOutput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a beginning transaction output object.
     *
     * @return a beginning transaction output object
     */
    public BeginTransactionOutput build() {
        return new BeginTransactionOutputImpl(this);
    }

    /**
     * Representation of the implement of BeginTransactionOutput.
     */
    private static final class BeginTransactionOutputImpl
            extends DefaultOutputImplBase
            implements BeginTransactionOutput {

        private final Map<Class<? extends BeginTransactionOutput>,
                BeginTransactionOutput> augmentation;

        private BeginTransactionOutputImpl(BeginTransactionOutputBuilder base) {
            message = base.message;
            resultCode = base.resultCode;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public int hashCode() {
            return Objects.hash(message, resultCode, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            BeginTransactionOutputImpl other =
                    (BeginTransactionOutputImpl) obj;
            return Objects.equals(message, other.getMessage()) &&
                    Objects.equals(resultCode, other.getResultCode()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("message", message)
                    .add("resultCode", resultCode)
                    .add("augmentation", augmentation).toString();
        }
    }
}