/*
 * Copyright 2017-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.condition.instance;

import org.onosproject.nemo.model.common.ConditionParameterName;
import org.onosproject.nemo.model.common.ConditionSegmentId;
import org.onosproject.nemo.model.operation.condition.instance.condition
        .segment.ConditionParameterTargetValue;

/**
 * The condition of an segment.
 */
public interface ConditionSegment {

    /**
     * Returns the id of condition segment.
     *
     * @return the id
     */
    ConditionSegmentId getConditionSegmentId();

    /**
     * Returns the parameter name of condition segment.
     *
     * @return the parameter name
     */
    ConditionParameterName getConditionParameterName();

    /**
     * Returns the match pattern of condition parameter.
     *
     * @return the match pattern
     */
    ConditionParameterMatchPattern getConditionParameterMatchPattern();

    /**
     * Returns the target value of condition parameter.
     *
     * @return the target value
     */
    ConditionParameterTargetValue getConditionParameterTargetValue();

    /**
     * Returns the operator of precursor relation.
     *
     * @return the operator
     */
    PrecursorRelationOperator getPrecursorRelationOperator();

    /**
     * Returns the order of the instance.
     *
     * @return the order
     */
    long getOrder();

    /**
     * Returns the condition segment key of the instance.
     *
     * @return the condition segment key
     */
    ConditionSegmentKey getKey();

    /**
     * Types of match pattern of condition parameter.
     */
    public enum ConditionParameterMatchPattern {

        /** less than valued pattern. */
        LessThan(0),

        /** not less than valued pattern. */
        NotLessThan(1),

        /** equal valued pattern. */
        Equal(2),

        /** not equal valued pattern. */
        NotEqual(3),

        /** greater than valued pattern. */
        GreaterThan(4),

        /** not greater than valued pattern. */
        NotGreaterThan(5),

        /** between valued pattern. */
        Between(6),

        /** Periodical valued pattern. */
        Periodical(7);

        private static final java.util.Map<Integer,
                ConditionParameterMatchPattern> VALUE_MAP;

        static {
            final com.google.common.collect.ImmutableMap.Builder<Integer,
                    ConditionParameterMatchPattern> b
                    = com.google.common.collect.ImmutableMap.builder();
            for (ConditionParameterMatchPattern enumItem :
                    ConditionParameterMatchPattern.values()) {
                b.put(enumItem.value, enumItem);
            }

            VALUE_MAP = b.build();
        }

        int value;

        private ConditionParameterMatchPattern(int value) {
            this.value = value;
        }

        /**
         * Returns the corresponding ConditionParameterMatchPattern
         * item from integer value.
         *
         * @param valueArg integer value
         * @return corresponding ConditionParameterMatchPattern item
         */
        public static ConditionParameterMatchPattern forValue(int valueArg) {
            return VALUE_MAP.get(valueArg);
        }

        /**
         * Returns the integer value.
         *
         * @return integer value
         */
        public int getIntValue() {
            return value;
        }
    }

    /**
     * Types of precursor relation operator.
     */
    public enum PrecursorRelationOperator {

        /** none valued operator. */
        None(0),

        /** and valued operator. */
        And(1),

        /** or valued operator. */
        Or(2),

        /** not than valued operator. */
        Not(3);

        private static final java.util.Map<Integer,
                PrecursorRelationOperator> VALUE_MAP;

        static {
            final com.google.common.collect.ImmutableMap.Builder<Integer,
                    PrecursorRelationOperator> b
                    = com.google.common.collect.ImmutableMap.builder();
            for (PrecursorRelationOperator enumItem :
                    PrecursorRelationOperator.values()) {
                b.put(enumItem.value, enumItem);
            }

            VALUE_MAP = b.build();
        }

        int value;

        private PrecursorRelationOperator(int value) {
            this.value = value;
        }

        /**
         * Returns the corresponding PrecursorRelationOperator
         * item from integer value.
         *
         * @param valueArg integer value
         * @return corresponding PrecursorRelationOperator item
         */
        public static PrecursorRelationOperator forValue(int valueArg) {
            return VALUE_MAP.get(valueArg);
        }

        /**
         * Returns the integer value.
         *
         * @return integer value
         */
        public int getIntValue() {
            return value;
        }
    }

}

