/*
 * Copyright 2017-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.condition.instance;

import org.onosproject.nemo.model.common.ConditionParameterName;
import org.onosproject.nemo.model.common.ConditionSegmentId;
import org.onosproject.nemo.model.operation.condition.instance.condition
        .segment.ConditionParameterTargetValue;

import static com.google.common.base.MoreObjects.toStringHelper;
import static org.onosproject.nemo.model.operation.action.instance
        .parameter.values.Utils.checkOrderRange;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * The description of builder for condition segment objects.
 */
public class ConditionSegmentBuilder {

    private Map<Class<? extends ConditionSegment>, ConditionSegment>
            augmentation = new HashMap<>();
    private ConditionSegment.ConditionParameterMatchPattern
            conditionParameterMatchPattern;
    private ConditionParameterName conditionParameterName;
    private ConditionParameterTargetValue conditionParameterTargetValue;
    private ConditionSegmentId conditionSegmentId;
    private ConditionSegmentKey key;
    private Long order;
    private ConditionSegment.PrecursorRelationOperator
            precursorRelationOperator;

    /**
     * Creates an instance by default.
     */
    public ConditionSegmentBuilder() {
    }

    /**
     * Creates a condition segment builder from a given condition segment.
     *
     * @param base condition segment
     */
    public ConditionSegmentBuilder(ConditionSegment base) {
        if (base.getKey() == null) {
            this.key = new ConditionSegmentKey(
                    base.getConditionSegmentId()
            );
            this.conditionSegmentId = base.getConditionSegmentId();
        } else {
            this.key = base.getKey();
            this.conditionSegmentId = key.getConditionSegmentId();
        }
        this.conditionParameterMatchPattern =
                base.getConditionParameterMatchPattern();
        this.conditionParameterName = base.getConditionParameterName();
        this.conditionParameterTargetValue =
                base.getConditionParameterTargetValue();
        this.order = base.getOrder();
        this.precursorRelationOperator = base.getPrecursorRelationOperator();
    }

    /**
     * Sets condition parameter match pattern to be used by the builder.
     *
     * @param value the condition parameter match pattern
     * @return self
     */
    public ConditionSegmentBuilder conditionParameterMatchPattern(
            ConditionSegment.ConditionParameterMatchPattern value) {
        this.conditionParameterMatchPattern = value;
        return this;
    }

    /**
     * Sets the condition parameter name to be used by the builder.
     *
     * @param value the condition parameter name
     * @return self
     */
    public ConditionSegmentBuilder
            conditionParameterName(ConditionParameterName value) {
        this.conditionParameterName = value;
        return this;
    }

    /**
     * Sets the condition parameter target value to be used by the builder.
     *
     * @param value the condition parameter target value
     * @return self
     */
    public ConditionSegmentBuilder conditionParameterTargetValue(
            ConditionParameterTargetValue value) {
        this.conditionParameterTargetValue = value;
        return this;
    }

    /**
     * Sets the condition segment id value to be used by the builder.
     *
     * @param value the condition segment id
     * @return self
     */
    public ConditionSegmentBuilder conditionSegmentId(
            ConditionSegmentId value) {
        this.conditionSegmentId = value;
        return this;
    }

    /**
     * Sets the condition segment key value to be used by the builder.
     *
     * @param value condition segment key value
     * @return self
     */
    public ConditionSegmentBuilder key(ConditionSegmentKey value) {
        this.key = value;
        return this;
    }

    /**
     * Sets the order to be used by the builder.
     *
     * @param value order
     * @return self
     */
    public ConditionSegmentBuilder order(long value) {
        checkOrderRange(value);
        this.order = value;
        return this;
    }

    /**
     * Sets the precursor relation operator to be used by the builder.
     *
     * @param value precursor relation operator
     * @return self
     */
    public ConditionSegmentBuilder precursorRelationOperator(
            ConditionSegment.PrecursorRelationOperator value) {
        this.precursorRelationOperator = value;
        return this;
    }

    /**
     * Adds the augmentation to be used by the builder.
     * Remove the augmentation, if the parameter value is null.
     *
     * @param augmentationType augmentation type
     * @param augmentation the augmentation to be used
     * @return self
     */
    public ConditionSegmentBuilder addAugmentation(
            Class<? extends ConditionSegment> augmentationType,
            ConditionSegment augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation to be used by parameter value builder.
     *
     * @param augmentationType remove augmentation
     * @return a parameter value builder
     */
    public ConditionSegmentBuilder removeAugmentation(
            Class<? extends ConditionSegment> augmentationType) {
            this.augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a new parameter value instance.
     *
     * @return the parameter value instance
     */
    public ConditionSegment build() {
        return new ConditionSegmentImpl(this);
    }

    /**
     * The implementation of condition segment to create an instance.
     */
    private static final class ConditionSegmentImpl
            implements ConditionSegment {

        private final ConditionParameterMatchPattern
                conditionParameterMatchPattern;
        private final ConditionParameterName conditionParameterName;
        private final ConditionParameterTargetValue
                conditionParameterTargetValue;
        private final ConditionSegmentId conditionSegmentId;
        private final ConditionSegmentKey key;
        private final Long order;
        private final PrecursorRelationOperator precursorRelationOperator;
        private Map<Class<? extends ConditionSegment>, ConditionSegment>
                augmentation = Collections.emptyMap();

        private ConditionSegmentImpl(ConditionSegmentBuilder base) {
            if (base.key == null) {
                this.key = new ConditionSegmentKey(
                        base.conditionSegmentId
                );
                this.conditionSegmentId = base.conditionSegmentId;
            } else {
                this.key = base.key;
                this.conditionSegmentId = key.getConditionSegmentId();
            }
            this.conditionParameterMatchPattern =
                    base.conditionParameterMatchPattern;
            this.conditionParameterName = base.conditionParameterName;
            this.conditionParameterTargetValue =
                    base.conditionParameterTargetValue;
            this.order = base.order;
            this.precursorRelationOperator =
                    base.precursorRelationOperator;
            this.augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public ConditionParameterMatchPattern
                getConditionParameterMatchPattern() {
            return conditionParameterMatchPattern;
        }

        @Override
        public ConditionParameterName getConditionParameterName() {
            return conditionParameterName;
        }

        @Override
        public ConditionParameterTargetValue
                getConditionParameterTargetValue() {
            return conditionParameterTargetValue;
        }

        @Override
        public ConditionSegmentId getConditionSegmentId() {
            return conditionSegmentId;
        }

        @Override
        public ConditionSegmentKey getKey() {
            return key;
        }

        @Override
        public long getOrder() {
            return order;
        }

        @Override
        public PrecursorRelationOperator getPrecursorRelationOperator() {
            return precursorRelationOperator;
        }

        @Override
        public int hashCode() {
            return Objects.hash(conditionParameterMatchPattern,
                    conditionParameterName, conditionParameterTargetValue,
                    conditionSegmentId, key, order, precursorRelationOperator,
                    augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ConditionSegment other = (ConditionSegment) obj;
            ConditionSegmentImpl otherImpl = (ConditionSegmentImpl) obj;
            return Objects.equals(conditionParameterMatchPattern,
                            other.getConditionParameterMatchPattern()) &&
                    Objects.equals(conditionParameterName,
                            other.getConditionParameterName()) &&
                    Objects.equals(conditionParameterTargetValue,
                            other.getConditionParameterTargetValue()) &&
                    Objects.equals(conditionSegmentId,
                            other.getConditionSegmentId()) &&
                    Objects.equals(key, other.getKey()) &&
                    Objects.equals(order, other.getOrder()) &&
                    Objects.equals(precursorRelationOperator,
                            other.getPrecursorRelationOperator()) &&
                    Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("conditionParameterMatchPattern",
                            conditionParameterMatchPattern)
                    .add("conditionParameterName", conditionParameterName)
                    .add("conditionParameterTargetValue",
                            conditionParameterTargetValue)
                    .add("conditionSegmentId", conditionSegmentId)
                    .add("key", key)
                    .add("order", order)
                    .add("precursorRelationOperator",
                            precursorRelationOperator)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }

}
